@@ -1,16 +1,20 @@
 package Message::Stack;
+BEGIN {
+  $Message::Stack::VERSION = '0.20';
+}
 use Moose;
 
+# ABSTRACT: Deal with a "stack" of messages
+
 use Carp qw(croak);
 use MooseX::Storage;
 use MooseX::Types::Moose qw(HashRef);
 use Message::Stack::Message;
 use Message::Stack::Types qw(MessageStackMessage);
 
-our $VERSION = '0.19';
-
 with 'MooseX::Storage::Deferred';
 
+
 has messages => (
     traits => [ 'Array' ],
     is => 'rw',
@@ -29,6 +33,7 @@ has messages => (
     }
 );
 
+
 sub add {
     my ($self, $message) = @_;
 
@@ -49,24 +54,28 @@ sub for_id {
     $self->for_msgid(@_);
 }
 
+
 sub for_msgid {
     my ($self, $msgid) = @_;
 
     return $self->search(sub { $_->msgid eq $msgid if $_->has_msgid });
 }
 
+
 sub for_level {
     my ($self, $level) = @_;
 
     return $self->search(sub { $_->level eq $level if $_->has_level });
 }
 
+
 sub for_scope {
     my ($self, $scope) = @_;
 
     return $self->search(sub { $_->scope eq $scope if $_->has_scope });
 }
 
+
 sub for_subject {
     my ($self, $subject) = @_;
 
@@ -78,6 +87,7 @@ sub has_id {
     $self->has_msgid(@_);
 }
 
+
 sub has_msgid {
     my ($self, $msgid) = @_;
 
@@ -86,6 +96,7 @@ sub has_msgid {
     return $self->for_msgid($msgid)->count ? 1 : 0;
 }
 
+
 sub has_level {
     my ($self, $level) = @_;
 
@@ -94,6 +105,7 @@ sub has_level {
     return $self->for_level($level)->count ? 1 : 0;
 }
 
+
 sub has_scope {
     my ($self, $scope) = @_;
 
@@ -102,6 +114,7 @@ sub has_scope {
     return $self->for_scope($scope)->count ? 1 : 0;
 }
 
+
 sub has_subject {
     my ($self, $subject) = @_;
 
@@ -110,6 +123,7 @@ sub has_subject {
     return $self->for_subject($subject)->count ? 1 : 0;
 }
 
+
 sub search {
     my ($self, $coderef) = @_;
 
@@ -117,42 +131,107 @@ sub search {
     return Message::Stack->new(messages => \@messages);
 }
 
+
+sub reset_scope {
+    my ($self, $scope) = @_;
+
+    return 0 unless $self->has_messages;
+
+    my $filtered;
+    foreach my $message (@{$self->messages}) {
+        next if($message->scope eq $scope);
+        push @{$filtered}, $message;
+    }
+
+    $self->messages($filtered);
+}
+
+
+sub reset_level {
+    my ($self, $level) = @_;
+
+    return 0 unless $self->has_messages;
+
+    my $filtered;
+    foreach my $message (@{$self->messages}) {
+        next if($message->level eq $level);
+        push @{$filtered}, $message;
+    }
+
+    $self->messages($filtered);
+}
+
+
+sub reset_msgid {
+    my ($self, $msgid) = @_;
+
+    return 0 unless $self->has_messages;
+
+    my $filtered;
+    foreach my $message (@{$self->messages}) {
+        next if($message->msgid eq $msgid);
+        push @{$filtered}, $message;
+    }
+
+    $self->messages($filtered);
+}
+
+
+sub reset_subject {
+    my ($self, $subject) = @_;
+
+    return 0 unless $self->has_messages;
+
+    my $filtered;
+    foreach my $message (@{$self->messages}) {
+        next if($message->subject eq $subject);
+        push @{$filtered}, $message;
+    }
+
+    $self->messages($filtered);
+}
+
+
 __PACKAGE__->meta->make_immutable;
 no Moose;
 1;
-
 __END__
+=pod
 
 =head1 NAME
 
 Message::Stack - Deal with a "stack" of messages
 
+=head1 VERSION
+
+version 0.20
+
 =head1 SYNOPSIS
 
-  my $stack = Message::Stack->new;
+    my $stack = Message::Stack->new;
 
-  $stack->add(Message::Stack::Message->new(
+    $stack->add(Message::Stack::Message->new(
       msgid     => 'something_happened',
       level     => 'error',
-      scope     => 'login_form',
+      scope     => 'login_formm',
       subject   => 'username',
       text      => 'Something happened!'
-  ));
-  # Or... for those that want to type less
-  $stack->add({
+    ));
+    # Or... for those that want to type less
+    $stack->add({
       msgid     => 'something_else_happened',
       level     => 'error',
       scope     => 'login_form',
       subject   => 'password',
       text      => 'Something else happened!'
-  });
-  
-  ...
-  my $errors = $stack->for_level($error);
-  # Or
-  my $login_form_errors = $stack->for_scope('login_form');
-  $login_form_errors->for_subject('username');
-  print "Username has ".$login_form_errors->count." errors.\n";
+    });
+
+    ...
+    my $errors = $stack->for_level($error);
+    # Or
+    my $login_form_errors = $stack->for_scope('login_form');
+    $login_form_errors->for_subject('username');
+    print "Username has ".$login_form_errors->count." errors.\n";
 
 =head1 DESCRIPTION
 
@@ -165,6 +244,8 @@ This is not a logging mechanism.  The original use was to store various errors
 or messages that occur during processing for later display in a web
 application.  The messages are added via C<add>.
 
+=head1 NOTES
+
 =head2 Note About msgid
 
 msgid used to be id.  It was renamed to be a bit more description.  All the
@@ -182,36 +263,43 @@ gist is:
   ...
   my $stack = Message::Stack->thaw($json, { format => 'JSON' });
 
-=head1 METHODS
-
-=head2 add ($message)
-
-Adds the supplied message to the stack.  C<$message> may be either a
-L<Message::Stack::Message> object or a hashref with similar keys.
-
-=head2 count
-
-Returns the number of messages in the stack.
+=head1 ATTRIBUTES
 
 =head2 messages
 
 Returns the full arrayref of messages for this stack.
 
+=head1 METHODS
+
+=head2 count
+
+Returns the number of messages in the stack.
+
 =head2 first_message
 
 Returns the first message (if there is one, else undef)
 
-=head2 search (CODEREF)
-
-Returns a Message::Stack containing messages that return true when passed
-to the coderef argument.
-
-  $stack->search( sub { $_[0]->id eq 'someid' } )
-
 =head2 get_message ($index)
 
 Get the message at the supplied index.
 
+=head2 has_messages
+
+Returns true if there are messages in the stack, else false
+
+=head2 last_message
+
+Returns the last message (if there is one, else undef)
+
+=head2 reset
+
+Clear all messages, resetting this stack.
+
+=head2 add ($message)
+
+Adds the supplied message to the stack.  C<$message> may be either a
+L<Message::Stack::Message> object or a hashref with similar keys.
+
 =head2 for_msgid ($msgid)
 
 Returns a new Message::Stack containing only the message objects with the
@@ -236,10 +324,6 @@ Returns a new Message::Stack containing only the message objects with the
 supplied subject. If there are no messages for that subject then the stack
 returned will have no messages.
 
-=head2 has_messages
-
-Returns true if there are messages in the stack, else false
-
 =head2 has_msgid ($msgid)
 
 Returns true if there are messages with the supplied msgid.
@@ -256,17 +340,28 @@ Returns true if there are messages with the supplied scope.
 
 Returns true if there are messages with the supplied subject.
 
-=head2 last_message
+=head2 search (CODEREF)
 
-Returns the last message (if there is one, else undef)
+Returns a Message::Stack containing messages that return true when passed
+to the coderef argument.
 
-=head2 reset
+  $stack->search( sub { $_[0]->id eq 'someid' } )
 
-Clear all messages, resetting this stack.
+=head2 reset_scope($scope)
 
-=head1 AUTHOR
+Clears the stack of all messages of scope $scope.
 
-Cory G Watson, C<< <gphat at cpan.org> >>
+=head2 reset_level($level)
+
+Clears the stack of all messages of level $level.
+
+=head2 reset_msgid($msgid)
+
+Clears the stack of all messages of msgid $msgid.
+
+=head2 reset_subject($subject)
+
+Clears the stack of all messages of subject $subject.
 
 =head1 CONTRIBUTORS
 
@@ -280,9 +375,18 @@ Jon Wright
 
 Mike Eldridge
 
-=head1 COPYRIGHT & LICENSE
+Andrew Nelson
 
-Copyright 2010 Cory G Watson, all rights reserved.
+=head1 AUTHOR
+
+Cory G Watson <gphat@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2011 by Cory G Watson.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
